import toml
import os
import sys


class ConfigFile:

    CONFIG_FILE_PATH = f'{os.environ["HOME"]}/.chpwd_cmd'

    def __init__(self):
        try:
            self.config = toml.load(open(self.CONFIG_FILE_PATH))
        except toml.decoder.TomlDecodeError as e:
            sys.stderr.write(f'configuration file ({self.CONFIG_FILE_PATH}) seems to be wrong\n {e}')
            raise ConfigFileError(e) from None
        except FileNotFoundError as e:
            sys.stderr.write(f'configuration file ({self.CONFIG_FILE_PATH}) does not exist\n {e}')
            raise ConfigFileError(
                f'configuration file ({self.CONFIG_FILE_PATH}) does not exist {e}') from None


class ConfigFileError(Exception):
    pass



import sys
import glob
import pathlib
import subprocess as subproc
from schema import Schema, SchemaError
from chpwd_cmd.color import green, red


class WorkDir:

    schema = Schema({'dirpath': str, 'cmd': str})

    def __init__(self, name, workdir, cwd):
        self.name = name
        try:
            self.schema.validate(workdir)
        except SchemaError as e:
            raise WorkDirInvalid(e) from None

        self.cmd = workdir['cmd']
        p = pathlib.Path(workdir['dirpath'])

        self.dirpath_candidates = glob.glob(str(p.expanduser()))
        self.enable = True if cwd in self.dirpath_candidates else False
        self.cwd = cwd

    def __execute__(self):
        try:
            subproc.run(self.cmd, cwd=self.cwd, shell=True, check=True)
        except subproc.CalledProcessError as e:
            sys.stderr.write(
                f'{red(self.cmd)} in workdir[{red(self.name)}]\
                    was not successful\n')
            raise (e)

        sys.stdout.write(
            f'{green(self.cmd)} in workdir[{green(self.name)}] executed by chpwd_cmd\n')

    def __dryrun__(self):
        if self.enable:
            sys.stdout.write(
                f'{green(self.cmd)} in workdir[{green(self.name)}] execute\n')
        else:
            sys.stdout.write(
                f'{red(self.cmd)} in workdir[{red(self.name)}] not execute\n')

    def __listdir__(self):
        sys.stdout.write(
            f'workdir[{green(self.name)}] {len(self.dirpath_candidates)} directory paths\n')
        sorted_dirpath = sorted(self.dirpath_candidates)
        for dir in sorted_dirpath:
            sys.stdout.write(f'{dir}\n')

    def execute(self, mode):
        if self.enable and mode == 'exec':
            self.__execute__()
        elif mode == 'dryrun':
            self.__dryrun__()
        elif mode == 'listdir':
            self.__listdir__()


class WorkDirInvalid(Exception):
    pass



from nose.tools import assert_equal, with_setup, raises
import sys
import chpwd_cmd.arguments as args


class TestArguments:

    def setup(self):
        # remove parameter with nosetests
        del sys.argv[1:]

    @with_setup(setup=setup)
    def test_mode_listdir_cwd_none_success(self):
        sys.argv.extend(['-m', 'listdir'])
        res = args.Arguments().get()
        assert_equal(res.mode, 'listdir')
        assert_equal(res.cwd, None)

    @with_setup(setup=setup)
    def test_mode_listdir_cwd_success(self):
        sys.argv.extend(['-m', 'listdir', '-c', '/home/dir'])
        res = args.Arguments().get()
        assert_equal(res.mode, 'listdir')
        assert_equal(res.cwd, '/home/dir')

    @with_setup(setup=setup)
    def test_mode_none_cwd_success(self):
        sys.argv.extend(['-c', '/home/dir'])
        res = args.Arguments().get()
        assert_equal(res.mode, 'exec')
        assert_equal(res.cwd, '/home/dir')

    @raises(args.ArgumentsError)
    @with_setup(setup=setup)
    def test_mode_exec_cwd_none_value_error(self):
        sys.argv.extend(['-m', 'exec'])
        args.Arguments().get()

    @raises(args.ArgumentsError)
    @with_setup(setup=setup)
    def test_mode_dryrun_cwd_none_value_error(self):
        sys.argv.extend(['-m', 'dryrun'])
        args.Arguments().get()

    @raises(args.ArgumentsError)
    @with_setup(setup=setup)
    def test_no_command_line_arguments(self):
        args.Arguments().get()



from nose.tools import raises

import chpwd_cmd.workdir as wd


class TestWorkdir:

    # error
    @raises(wd.WorkDirInvalid)
    def test_type_invalid(self):
        wd.WorkDir('test', {'dirpath': 0, 'cmd': []}, 'pwd')

    @raises(wd.WorkDirInvalid)
    def test_dirpath_key_missing(self):
        wd.WorkDir('test', {'cmd': 'pwd'}, 'pwd')

    @raises(wd.WorkDirInvalid)
    def test_cmd_key_missing(self):
        wd.WorkDir('test', {'dirpath': '/home'}, 'pwd')

    @raises(wd.WorkDirInvalid)
    def test_all_key_missing(self):
        wd.WorkDir('', {}, '')



import sys
from chpwd_cmd.workdir import WorkDir
from chpwd_cmd.config_file import ConfigFile, ConfigFileError
from chpwd_cmd.arguments import Arguments, ArgumentsError


def main():
    try:
        workdirs = ConfigFile().config['workdir']
        args = Arguments().get()
    except ConfigFileError:
        sys.exit()
    except ArgumentsError:
        Arguments().help()
        sys.exit()

    for workdir_name in workdirs:
        wd = WorkDir(workdir_name, workdirs[workdir_name], args.cwd)
        wd.execute(args.mode)


if __name__ == "__main__":
    main()



import argparse


class Arguments:

    def __init__(self):
        self.parser = argparse.ArgumentParser(
            description='This program executes the commands in the \
                        configuration file according to the\
                        current working directory')
        self.parser.add_argument(
            '-c', '--cwd', type=str,
            help='current working directory')
        self.parser.add_argument(
            '-m', '--mode', default='exec',
            choices=['exec', 'dryrun', 'listdir'],
            help='running mode (default:exec)')

    def get(self):
        if not self.parser.parse_args().mode == 'listdir' and not self.parser.parse_args().cwd:
            raise ArgumentsError(
                f'command line arguments was invalid {self.parser.parse_args()}')
        return self.parser.parse_args()

    def help(self):
        self.parser.print_help()


class ArgumentsError(ValueError):
    pass



from nose.tools import raises
import chpwd_cmd.config_file as cf


class TestConfigFile:

    @raises(cf.ConfigFileError)
    def test_config_file_not_found(self):
        cf.ConfigFile.CONFIG_FILE_PATH = 'test/.notfound_test_chpwd_cmd'
        cf.ConfigFile()

    @raises(cf.ConfigFileError)
    def test_config_file_invalid(self):
        cf.ConfigFile.CONFIG_FILE_PATH = 'test/.test_invalid_config_file'
        cf.ConfigFile()



from setuptools import setup, find_packages

with open('requirements.txt') as requirements_file:
    install_requirements = requirements_file.read().splitlines()

with open('README.md') as readme:
    long_description = readme.read()

setup(
    name='chpwd_cmd',
    version='0.0.1',
    description='Execute the commands in the configuration file according to the current working directory.',
    author_email='teddy@sfc.wide.ad.jp',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='KIMOTO Mizuki',
    url='https://github.com/teddyyy/chpwd_cmd',
    license='MIT',
    packages=find_packages(),
    install_requires=install_requirements,
    entry_points={
        'console_scripts': [
            'chpwd_cmd=chpwd_cmd.chpwd_cmd:main',
        ]
    },
    classifiers=[
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8'
    ]
)



class Color:
    GREEN = '\033[32m'
    RED = '\033[31m'
    RESET = '\033[0m'


def green(text):
    return f'{Color.GREEN}{text}{Color.RESET}'


def red(text):
    return f'{Color.RED}{text}{Color.RESET}'



from nose.tools import assert_equal, with_setup, raises
import sys
import os
import subprocess
from io import StringIO
from contextlib import redirect_stdout

from chpwd_cmd import chpwd_cmd
from chpwd_cmd.color import green, red


def run_chpwd_cmd():
    io = StringIO()
    with redirect_stdout(io):
        chpwd_cmd.main()

    return io.getvalue()


class TestChpwdCmd:

    cwd = os.getcwd()

    def setup(self):
        # remove parameter with nosetests
        del sys.argv[1:]
        chpwd_cmd.ConfigFile.CONFIG_FILE_PATH = 'test/files/.test_chpwd_cmd'

    # error
    @raises(SystemExit)
    @with_setup(setup=setup)
    def test_no_configuration(self):
        chpwd_cmd.ConfigFile.CONFIG_FILE_PATH = 'test/files/.no_chpwd_cmd'
        sys.argv.extend(['-c', self.cwd, '-m', 'exec'])
        chpwd_cmd.main()

    @raises(SystemExit)
    @with_setup(setup=setup)
    def test_no_commmand_line_args(self):
        sys.argv.extend([])
        chpwd_cmd.main()

    # exec
    @with_setup(setup=setup)
    def test_chpwd_exec_success(self):
        sys.argv.extend(['-c', self.cwd, '-m', 'exec'])
        assert_equal(run_chpwd_cmd(
        ), f"{green('pwd')} in workdir[{green('test')}] executed by chpwd_cmd\n")

    @with_setup(setup=setup)
    def test_chpwd_exec_home_dir_success(self):
        sys.argv.extend(['-c', os.environ['HOME'], '-m', 'exec'])
        assert_equal(run_chpwd_cmd(
        ), f"{green('pwd')} in workdir[{green('test_home_dir')}] executed by chpwd_cmd\n")

    @with_setup(setup=setup)
    def test_chpwd_exec_none_output(self):
        sys.argv.extend(['-c', f'{self.cwd}/test/testdirnone', '-m', 'exec'])
        assert_equal(run_chpwd_cmd(), '')

    @raises(subprocess.CalledProcessError)
    @with_setup(setup=setup)
    def test_chpwd_exec_command_exception(self):
        chpwd_cmd.ConfigFile.CONFIG_FILE_PATH = 'test/files/.test_command_exception'
        sys.argv.extend(['-c', '.', '-m', 'exec'])

        assert_equal(run_chpwd_cmd(
        ), f"{red('aaaaa')} in workdir[{red('cmd_exception')}] was not successful\n")

    # dryrun
    @with_setup(setup=setup)
    def test_chpwd_dryrun_success(self):
        sys.argv.extend(['-c', os.environ['HOME'], '-m', 'dryrun'])
        lines = run_chpwd_cmd().splitlines()
        assert_equal(
            lines[0], f"{red('pwd')} in workdir[{red('test')}] not execute")
        assert_equal(
            lines[1], f"{green('pwd')} in workdir[{green('test_home_dir')}] execute")
        assert_equal(
            lines[2], f"{red('pwd')} in workdir[{red('wildcard')}] not execute")

    # listdir
    @with_setup(setup=setup)
    def test_chpwd_listdir_success(self):
        sys.argv.extend(['-m', 'listdir'])
        lines = run_chpwd_cmd().splitlines()
        assert_equal(
            lines[0], f"workdir[{green('test')}] 1 directory paths")
        assert_equal(lines[1], self.cwd)
        assert_equal(
            lines[2], f"workdir[{green('test_home_dir')}] 1 directory paths")
        assert_equal(lines[3], os.environ["HOME"])
        assert_equal(
            lines[4], f"workdir[{green('wildcard')}] 2 directory paths")
        assert_equal(
            lines[5], f"{self.cwd}/test/testdir/dir1")
        assert_equal(
            lines[6], f"{self.cwd}/test/testdir/dir2")
